unit MMJEdit;

interface
{ TMMJEdit component is TEdit with added features/propreties:

   --> CapitalizeFirst  (it sets the first letter to capital)
   --> EditFunction (Edit may be unchanged, accept only numbers, or have additional
                  spin buttons that act accordingly to displayed number value in the box,
                  and vice versa.)
   --> EditType (Property which affects drawing of component)
   --> EnterAsTab (Self-explanatory)

   Happy programming
   Mihaela Mihaljevic Jakic
   mickj@hi.hinet.hr

}
uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls;
type
  TLanguage=(English,Croatian);
  TEditType=(etDefault,etFancy);
  TEditFunction=(efDefault,efNumbersOnly,efUpDown);
  TMouseType=(Up,Down,Moving,None);
  TMMJEdit = class(TCustomEdit)
  private
    { Private declarations }
    FPressed:Boolean;
    FPressedUp:Boolean;
    FPressedDown:Boolean;
    FMouse:TMouseType;
    FEditType:TEditType;
    FIncrement:integer;
    FEditFunction:TEditFunction;
    FLanguage:TLanguage;
    FFocusColor: TColor;
    FFrameColor: TColor;
    FFlatColor: TColor;
    FCapitalizeFirst:Boolean;
    FEnterAsTab:Boolean;
    FAlignCenter:Boolean;
    FNoBorders:Boolean;
    FUpDownColor:TColor;
    procedure SetCapitalizeFirst(Value:Boolean);
    procedure SetEnterAsTab(Value:Boolean);
    procedure SetNoBorders(Value:Boolean);
    procedure SetAlignCenter(Value:Boolean);
    procedure SetFrameColor(Value:TColor);
    procedure SetFocusColor(Value:TColor);
    procedure SetFlatColor(Value:TColor);
    procedure SetUpDownColor(Value:TColor);
    procedure SetLanguage(Value :TLanguage);
    procedure SetEditType(Value :TEditType);
    procedure SetIncrement(Value :integer);
    procedure SetEditFunction(Value :TEditFunction);
    procedure DrawEditBox;
    procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
    procedure CMMouseExit(var Message: TMessage); message CM_MOUSELEAVE;
  //  procedure WMMouseMove(var Msg:TMessage); message wm_MouseMove;
    procedure WMPaint(var Message : TMessage); message WM_PAINT;
    procedure KillFocus(var Message : TWMKillFocus); message WM_KILLFOCUS;
    procedure SetFocus (var Message: TWMSetFocus); message WM_SETFOCUS;
    procedure WMLBUTTONDOWN(var Message: TWMMouse); message WM_LBUTTONDOWN; 
    procedure WMLBUTTONUP(var Message: TWMMouse); message WM_LBUTTONUP; 

    procedure CreateWnd; override;
    procedure Click; override;
    procedure DrawUpDown(Focus:Boolean);
  protected
    { Protected declarations }
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure KeyPress(var Key: Char); override;
    procedure CreateParams(var Params: TCreateParams); override;
    procedure EnlargeRect(Wid,Hei:integer);
  published
    { Published declarations }
    property CapitalizeFirst:Boolean
       read FCapitalizeFirst
       write SetCapitalizeFirst;
    property NoBorders:Boolean
       read FNoBorders
       write SetNoBorders;
    property EnterAsTab:Boolean
       read FEnterAsTab
       write SetEnterAsTab;
    property AlignCenter:Boolean
       read FAlignCenter
       write SetAlignCenter;
    property FrameColor:TColor
       read FFrameColor
       write SetFrameColor;
    property UpDownColor:TColor
       read FUpDownColor
       write SetUpDownColor;
    property FocusColor:TColor
       read FFocusColor
       write SetFocusColor;
    property FlatColor:TColor
       read FFlatColor
       write SetFlatColor;
    property Language:TLanguage
       read FLanguage
       write SetLanguage;
    property EditType:TEditType
       read FEditType
       write SetEditType;
    property Increment:integer
       read FIncrement
       write SetIncrement;
    property EditFunction:TEditFunction
       read FEditFunction
       write SetEditFunction;
    property ParentColor;
    property OnChange;
    property AutoSelect;
    property Anchors;
    property Align;
    property BiDiMode;
    property CharCase;
    property Constraints;
    property DragCursor;
    property DragKind;
    property DragMode;
    property Enabled;
    property Font;
    property HideSelection;
    property ImeMode;
    property ImeName;
    property MaxLength;
    property OEMConvert;
    property ParentBiDiMode;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PasswordChar;
    property PopupMenu;
    property ReadOnly;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Text;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDock;
    property OnStartDrag;
  end;
procedure Register;
const
   UpDownWidth=15;
implementation

constructor TMMJEdit.Create(AOwner:TComponent);
begin
   inherited Create(AOwner);
   FLanguage:=English;
   FFrameColor:=$003D3D7A;
   FFocusColor:=$00B0D8FF;
   FFlatColor:=$00BFFFFF;
   Color:=FFlatColor;
   FPressed:=False;
   FPressedUp:=False;
   FPressedDown:=False;
   FMouse:=none;
   AutoSize := False;
   Ctl3D := False;
   BorderStyle := bsNone;
   ControlStyle := ControlStyle - [csFramed];
   FCapitalizeFirst:=True;
   FEnterAsTab:=True;
   FIncrement:=1;
   FAlignCenter:=False;
   FNoBorders:=False;
   FEditType:=etFancy;
   FEditFunction:=efDefault;
   FUpDownColor:=clNavy;
   //ClientHeight:=ClientHeight+1;
end;

procedure TMMJEdit.CreateParams(var Params: TCreateParams);
begin
   inherited CreateParams(Params);
   if FAlignCenter then
      Params.Style:=Params.Style or ES_CENTER;
   if FEditFunction <>efDefault then
      Params.Style:= Params.Style or ES_NUMBER;
end;

procedure TMMJEdit.WMLBUTTONDOWN(var Message: TWMMouse);
var
   br:integer;
begin
  inherited;
  FMouse:=moving;
  if FEditFunction<> efUpDown then
     Cursor:=crIBeam;
  if FEditFunction=efUpDown then begin
     //draw button in pressed state...
       br:=StrToInt(Text);
     if (message.xpos >= ClientWidth-UpDownWidth+1) and
        (message.YPos<=ClientHeight div 2) then begin
      //  FPressed := true;
        FPressedUp:=True;
        Text:=IntToStr(br+FIncrement);
        Refresh;
     end;
     if (message.xpos >= ClientWidth-UpDownWidth+1) and
        (message.YPos>=ClientHeight div 2) then begin
      //  FPressed := true;
        FPressedDown:=True;
        Text:=IntToStr(br-FIncrement);
        Refresh;
     end;
     Refresh;
  end;
end;

procedure TMMJEdit.WMLBUTTONUP(var Message: TWMMouse);
begin
  inherited;
  FMouse:=moving;
  if FEditFunction=efUpDown then begin
     //draw button in non-pressed state...
     if FPressed then begin
        FPressed := false;
        //Refresh;
     end;
     if FPressedUp then begin
        FPressedUp := false;
     //   Refresh;
     end;
     if FPressedDown then begin
        FPressedDown := false;
     //   Refresh;
     end;
     Refresh;
  end;
end;

procedure TMMJEdit.CreateWnd;
var
   R:TRect;
begin
  inherited CreateWnd;
  Perform(EM_SETMARGINS , EC_LEFTMARGIN, 4 );
  if FEditFunction=efUpDown then
     Perform(EM_SETMARGINS,EC_RIGHTMARGIN,(UpDownWidth+2) shl 16);
end; 

procedure TMMJEdit.EnlargeRect(Wid,Hei:integer);
var
   R:TRect;
begin
   Perform(EM_GETRECT,0,Integer(@R));
   InflateRect(R,Wid,Hei);
   Perform(EM_SETRECT , 0, Integer(@R) );
end;

destructor TMMJEdit.Destroy;
begin
   inherited Destroy;
end;

procedure TMMJEdit.SetLanguage(Value :TLanguage);
begin
   if FLanguage<> Value then begin
      FLanguage:=Value;
   end;
end;

procedure TMMJEdit.SetIncrement(Value :integer);
begin
   if FIncrement<>Value then
      FIncrement:=Value;
end;

procedure TMMJEdit.Click;
var 
  pt: TPoint;
  br:integer;
begin
   if FEditFunction=efUpDown then begin
      GetCursorPos(pt);
      if PtInRect(Rect(clientwidth-UpDownWidth+1,0,clientwidth,clientheight),
       ScreenToClient(pt)) then
          inherited Click;
   end
   else
      inherited Click;
end; 


procedure TMMJEdit.SetEditType(Value :TEditType);
begin
   if FEditType<> Value then begin
      FEditType:=Value;
   if FEditType=etDefault then begin
      BorderStyle := bsSingle;
      ControlStyle := ControlStyle + [csFramed];
   end;
   if FEditType=etFancy then begin
      BorderStyle := bsNone;
      ControlStyle := ControlStyle - [csFramed];
   end;
      RecreateWnd;
   end;
end;

procedure TMMJEdit.SetEditFunction(Value :TEditFunction);
begin
   if FEditFunction<> Value then begin
      FEditFunction:=Value;
      if FEditFunction<>efDefault then
         Text:='0';
      RecreateWnd;
   end;
end;

procedure TMMJEdit.SetFrameColor(Value:TColor);
begin
   if FFrameColor<>Value then
      FFrameColor:=Value;
   DrawEditBox;
end;

procedure TMMJEdit.SetUpDownColor(Value:TColor);
begin
   if FUpDownColor<>Value then
      FUpDownColor:=Value;
   DrawEditBox;
end;

function CroUpCase(c:char):char;
const
   aNasiMali: array[1..31 ] of Char =('a','b','c','','','d','','e','f','g','h','i','j','k',
             'l','m','n','o','p','q','r','s','','t','u','v','w','x',
             'y','z','');
   aNasiVeliki: array[1..31 ] of Char =('A','B','C','','','D','','E','F','G','H','I','J','K',
             'L','M','N','O','P','Q','R','S','','T','U','V','W','X',
             'Y','Z','');
var
   i:integer;
begin
   for i:= 1 to Length(aNasiMali) do begin
      if c = aNasiMali[i] then
         result:=aNasiVeliki[i];
   end;

end;

procedure TMMJEdit.KeyPress(var Key:Char);
const
   NasiMali=['a','b','c','','','d','','e','f','g','h','i','j','k',
             'l','m','n','o','p','q','r','s','','t','u','v','w','x',
             'y','z',''];
var
   MYForm: TForm;
begin
   if FEnterAsTab then begin
      if Key = #13 then begin
         MYForm := TForm(GetParentForm(Self));
         if not (MYForm = nil ) then
            SendMessage(MYForm.Handle, WM_NEXTDLGCTL, 0, 0);
         Key := #0;
      end 
      else if not FCapitalizeFirst then
     // if Key &tl;&gl; #0 then
         inherited KeyPress(Key);
   end;

   if  FCapitalizeFirst then begin
      inherited KeyPress(Key);
      if (SelStart=0) or (Text[SelStart]=' ') then begin
         if FLanguage=English then begin
            if Key in ['a'..'z'] then
               Key:=UpCase(key);
         end;
         if FLanguage=Croatian then begin
            if Key in NasiMali then
               Key:=CroUpCase(key);
         end;
      end;
   end;
end;


procedure TMMJEdit.KeyDown(var Key: Word; Shift: TShiftState);
var
   MYForm: TForm;
   CtlDir: Word;
begin
   if FEditFunction=efUpDown then begin
     //respond to up or down arrow keys or Return key with OnClick event if 
  //"ClickOnArrow" or "ClickOnReturn" property set... 
      inherited KeyDown(Key, Shift);
      if ((Key = vk_Down) or (Key = vk_Up))and (Shift = []) then begin
         Key := 0;
         inherited Click;
      end;
   end;

   if not FEnterAsTab then begin
      inherited;
      Exit;
   end;
   if (Key = VK_UP) or (Key = VK_DOWN) then
   begin
       MYForm := TForm(GetParentForm( Self ));
       if Key = VK_UP then CtlDir := 1
       else CtlDir :=0;
       if not (MYForm = nil ) then
           SendMessage(MYForm.Handle, WM_NEXTDLGCTL, CtlDir, 0);
   end
   else inherited KeyDown(Key, Shift);

end;


procedure TMMJEdit.SetFocusColor(Value:TColor);
begin
   if FFocusColor<>Value then
      FFocusColor:=Value;
   DrawEditBox;
end;

procedure TMMJEdit.SetFlatColor(Value:TColor);
begin
   if FFlatColor<>Value then
      FFlatColor:=Value;
   DrawEditBox;
end;

procedure TMMJEdit.CMMouseEnter(var Message: TMessage);
  begin
    Cursor:=crHandPoint;
    FMouse:=moving;
    if FEditType=etFancy then
       DrawEditBox;
  end;

procedure TMMJEdit.CMMouseExit(var Message: TMessage);
begin
    FMouse:=none;
    if FEditType=etFancy then
       DrawEditBox;
end;

procedure TMMJEdit.SetFocus (var Message: TWMSetFocus);
var
   hEdit:hWnd;
begin
   inherited;
   FMouse:=Moving;
 { if not(csDesigning in ComponentState) then }
 //if FEditType=etFancy then
  //     DrawEditBox;
  Invalidate;
end;

procedure TMMJEdit.KillFocus (var Message: TWMKillFocus);
begin
    inherited;
    FMouse:=none;
    //if FEditType=etFancy then
    //   DrawEditBox;
    Invalidate;
end;

procedure TMMJEdit.SetCapitalizeFirst(Value:Boolean);
begin
   if FCapitalizeFirst<>Value then begin
      FCapitalizeFirst:=Value;
   end;
end;

procedure TMMJEdit.SetEnterAsTab(Value:Boolean);
begin
   if FEnterAsTab<>Value then
      FEnterAsTab:=Value;
end;

procedure TMMJEdit.SetAlignCenter(Value:Boolean);
begin
   if FAlignCenter<>Value then
      FAlignCenter:=Value;
end;

procedure TMMJEdit.SetNoBorders(Value:Boolean);
begin
   if FNoBorders<>Value then
      FNoBorders:=Value;
   RecreateWnd;
end;

procedure TMMJEdit.WMPaint(var Message : TMessage);
begin
   inherited ;
   if FEditType=etFancy then
       DrawEditBox;
end;

procedure TMMJEdit.DrawUpDown(Focus:Boolean);
var
  dc: HDC;
  CntrPt: TPoint;
  ArrowUp : array [0..2] of TPoint;
  ArrowDown : array [0..2] of TPoint;
  clH:integer;
begin
   clH:=ClientHeight;
   if not Focus then
      clH:=ClientHeight-6;
   if FEditFunction=efUpDown then begin
      dc := getDC(handle);
      CntrPt := point(ClientWidth - UpDownWidth div 2, clH div 2);
      if FPressedUp or FPressedDown then
         CntrPt := point(CntrPt.x+1,CntrPt.y+1);

      ArrowUp[0] := point( CntrPt.x-3,CntrPt.y-3);
      ArrowUp[1] := point( CntrPt.x,CntrPt.y-6);
      ArrowUp[2] := point( CntrPt.x+3, CntrPt.y-3);

      ArrowDown[0] := point( CntrPt.x-3,CntrPt.y+3);
      ArrowDown[1] := point( CntrPt.x,CntrPt.y+6);
      ArrowDown[2] := point( CntrPt.x+3, CntrPt.y+3);

      with TCanvas.Create do begin
         Handle := dc;
         Brush.Color := clBtnFace;
      if FPressedUp or FPressedDown then
            Pen.color := clBtnShadow
         else
            Pen.color := clBtnHighlight;

         Moveto(clientwidth-UpDownWidth+2,clH-1);
         Lineto(clientwidth-UpDownWidth+2,1);
         Lineto(clientwidth-1,1);

         if FPressedUp or FPressedDown then
            Pen.color := clBtnHighlight
         else
            Pen.color := clBtnShadow;
         Lineto(clientwidth-1,clH-1);
         Lineto(clientwidth-UpDownWidth+2,clH-1);
          //draw the arrows...
         Brush.Color := FUpDownColor;
         Pen.color := clBlack;

         Polygon(ArrowUp);
         Polygon(ArrowDown);
         Handle := 0;
      end;
      ReleaseDC(handle,dc); 
   end;//UpDown
end;

procedure TMMJEdit.DrawEditBox;
var
   DC : HDC;
   R : TRect;
  BtnFaceBrush, WindowBrush, FocusBrush: HBRUSH;
   wCanvas:TCanvas;
   MouseIn : Boolean;
begin
    MouseIn:= FMouse<>none;
    wCanvas:=TCanvas.Create;
    wCanvas.handle:=GetWindowDC(Handle);
    GetWindowRect(Handle, R);
  try
    OffsetRect(R, -R.Left, -R.Top);
    BtnFaceBrush := CreateSolidBrush(ColorToRGB(FFrameColor));
    WindowBrush := CreateSolidBrush(ColorToRGB(FFlatColor));
    FocusBrush := CreateSolidBrush(ColorToRGB(FFocusColor));
    if Self.Focused or (FMouse = moving) then begin
       //or (MouseIn) and (not(Screen.ActiveControl is TMMJEdit))then begin
       DrawUpDown(True);
       Color := FFocusColor;
       if not FNoBorders then begin
          BorderStyle:=bsSingle;
          DrawEdge(wCanvas.handle,R,EDGE_RAISED,BF_RECT);
       end;
    end
    else begin
       DrawUpDown(False);
       Color := FFlatColor;
       if not FNoBorders then begin
          BorderStyle:=bsNone;
          InflateRect(R, -1, -4);
          FrameRect(DC, R, BtnFaceBrush);
          DrawEdge(wCanvas.handle,R,EDGE_BUMP,BF_BOTTOM + BF_LEFT);
       end;
    end;
  finally
    ReleaseDC(Handle, DC);
  end;
  DeleteObject(WindowBrush);
  DeleteObject(BtnFaceBrush);
  DeleteObject(FocusBrush);
end;

procedure Register;
begin
  RegisterComponents('Mihaela''s', [TMMJEdit]);
end;
end.
